using System.Collections.Generic;
using System.Linq;
using System;
using spike.Data.PersistentData;
using Newtonsoft.Json;
using System.Text;
using System.IO;
using spike.Utility.FileManaging;
using spike.Data;


namespace spike.Utility.FileManaging
{
    public enum MoveTypes
    {
        all,
        official,
        custom
    }

    public class FileManager
    {
        private String _filePathRelative;
        private String _fullPath;

        public FileManager(string fileName)
        {
            _filePathRelative = fileName;
            _fullPath = System.IO.Directory.GetCurrentDirectory() + _filePathRelative.Substring(1);
        }

        public static List<Move> PopulateList(ulong serverID, MoveTypes type)
        {

            if (type == MoveTypes.all)
            {
                List<Move> moveList = null;
                var list1 = MoveListManager.Instance.GetMoveList();
                FileManager handler2 = new FileManager(serverID + "-moves.json");
                var list2 = handler2.GetMoveListData();
                if (list2 == null)
                {
                    moveList = list1.moves;
                }
                else
                {
                    moveList = list1.moves.Concat(list2.moves).ToList();
                }
                return moveList.OrderBy(o => o.longName).ToList();
            }
            else if (type == MoveTypes.custom)
            {
                FileManager handler = new FileManager(serverID + "-moves.json");
                var list = handler.GetMoveListData();
                return list.moves.OrderBy(o => o.longName).ToList();
            }
            else if (type == MoveTypes.official)
            {
                var list = MoveListManager.Instance.GetMoveList();
                return list.moves.OrderBy(o => o.longName).ToList();
            }
            else
            {
                return null;
            }
        }

        public String GetPath() { return _filePathRelative; }
        public String GetAbsolutePath() { return _fullPath; }
        public bool FileExists()
        {
            return System.IO.File.Exists(_fullPath);
        }

        public void CreateFile()
        {
            if (FileExists())
            {
                throw new Exception($"Trying to create {_fullPath} even though file exists..");
            }

            try
            {
                System.IO.File.Create(_fullPath).Close();
            }
            catch (Exception e)
            {
                throw new Exception($"Trying to create {_fullPath} : {e.Message}");
            }
        }

        public void DeleteFile()
        {
            try
            {
                if (FileExists())
                {
                    System.IO.File.Delete(_fullPath);
                    Console.WriteLine($"Removed {_fullPath}");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine($"Tried to remove {_fullPath} but got error:\n{e.Message}");
            }
        }

        public String ReadFile()
        {
            StringBuilder sb = new StringBuilder();
            try
            {
                // Open the file to read from.
                using (StreamReader sr = File.OpenText(GetPath()))
                {
                    string s;
                    while ((s = sr.ReadLine()) != null)
                    {
                        sb.Append(s);
                    }
                }
            }
            catch (Exception e)
            {
                // Let the user know what went wrong.
                Console.WriteLine($"The file {_filePathRelative} could not be read:");
                Console.WriteLine(e.Message);
            }
            if (sb.ToString().Length == 0)
            {
                Console.WriteLine($"The file {_filePathRelative} Was empty ...");
            }

            return sb.ToString();
        }

        public MoveList GetMoveListData()
        {
            String JSONData = ReadFile();
            if (JSONData.Length <= 0)
            {
                Console.WriteLine($"File for {_filePathRelative} was empty ... Skipping!");
                return null;
            }
            MoveList content = JsonConvert.DeserializeObject<MoveList>(JSONData);
            return content;
        }

        public static void SaveMoves(MoveList output, MoveTypes type, ulong serverID = 0)
        {
            if (type == MoveTypes.custom && serverID == 0)
            {
                Console.WriteLine("You tried to save a Custom List without a Server ID. Shame.");
                return;
            }
            var fileName = type == MoveTypes.official ? "moves.json" : serverID + "-moves.json";
            FileManager handler = new FileManager(fileName);
            String JSONData = JsonConvert.SerializeObject(output);
            handler.WriteFile(JSONData);
        }

        public bool WriteFile(String content)
        {
            try
            {
                using (StreamWriter sw = File.CreateText(_filePathRelative))
                {
                    sw.WriteLine(content);
                    sw.Close();
                }
            }
            catch (Exception e)
            {
                // Let the user know what went wrong.
                Console.WriteLine($"The file {_filePathRelative} could not be written:");
                Console.WriteLine(e.Message);
                return false;
            }
            return true;
        }
    }
}
